#include "StdAfx.h"
#include "ComTools.h"

CComTools::CComTools(void)
{
	m_dispid = (-1);
	memset((LPVOID)&m_DispParams, 0, sizeof(DISPPARAMS));
	memset((LPVOID)&m_ExceptInfo, 0, sizeof(EXCEPINFO));
	clearError();
}

CComTools::~CComTools(void)
{
	clearDispParams();
	VariantClear(&m_vtReturnValue.GetVARIANT());
	clearError();
	ReleaseAll();
}

void CComTools::ReleaseAll()
{
	if (m_spTypeInfo.p != NULL)
	{
		m_spTypeInfo.Release(); 
	}
	if (m_spTypeInfo2.p != NULL)
	{
		m_spTypeInfo2.Release(); 
	}
	if (m_spDisp.p != NULL)
	{
		m_spDisp.Release(); 
	}
	if (m_spUnk.p != NULL)
	{
		m_spUnk.Release(); 
	}
}

void CComTools::clearError()
{
	m_dwErrCode = ERROR_SUCCESS;
	m_hr = S_OK;
	m_bstrErrMsg = bstr_t();
}

void CComTools::clearDispParams()
{
	if (m_DispParams.rgvarg != NULL)
	{
		for(int i = 0; i < (int)m_DispParams.cArgs; i++)
		{
			VariantClear(&m_DispParams.rgvarg[i]);
		}
		delete [] m_DispParams.rgvarg;
		m_DispParams.rgvarg = NULL;
	}
	m_DispParams.cArgs = 0; 
}

bool CComTools::GetErrorInfo(HRESULT& hr, bstr_t& bstrErrMsg, DWORD& dwErrCode)
{
	if (m_dwErrCode == ERROR_SUCCESS && m_hr == S_OK)
	{
		return false;
	}
	hr = m_hr;
	bstrErrMsg = m_bstrErrMsg.copy(); 
	dwErrCode = m_dwErrCode;
	return true;
}

void CComTools::TypeDescToDWord(const VARTYPE& vtFrom, VARTYPE *pvt, BOOL fIsRval)
{
	*pvt = 0;
	switch (vtFrom)
	{
		case VT_I2:
		case VT_I4:
		case VT_R4:
		case VT_R8:
		case VT_CY:
		case VT_BOOL:
		case VT_BSTR:
		case VT_DATE:
		case VT_DECIMAL:
		case VT_UI1:
		case VT_DISPATCH:
		case VT_USERDEFINED:
		case VT_VARIANT:
		case VT_VOID:
		// Elemental types signal end of type description
		(*pvt) |= vtFrom;
		break;

		case VT_PTR:
		// Pointers are byref (skip byref on return value) then recurse
		if (!fIsRval)
		{
			(*pvt) |= VT_BYREF;
		}
		TypeDescToDWord(vtFrom, pvt, FALSE);
		break;

		case VT_SAFEARRAY:
		// Arrays (safearray) get array attribute then recurse
		(*pvt) |= VT_ARRAY;
		TypeDescToDWord(vtFrom, pvt, FALSE);
		break;

		default:
		_ASSERT(0);
		break;
	}
}	

bool CComTools::CreateObject(LPCWSTR pszProgID)
{
	bool bRetVal = false;
	HRESULT hr = E_FAIL;
	clearError();
	try
	{
		if (m_spDisp.p != NULL)
		{
			return true;
		}
		TESTHR(hr = m_spUnk.CoCreateInstance(pszProgID, 0, CLSCTX_SERVER));
		TESTHR(hr = m_spUnk->QueryInterface(IID_IDispatch, (void**)&m_spDisp));
		bRetVal = (m_spDisp.p != NULL);
	}
	catch(_com_error& e)
	{
		if ((m_hr = e.Error()) != S_OK)
		{
			m_bstrErrMsg = e.ErrorMessage();
		}
		else
		{
			m_hr = ERR_UNEXPECTED_FOR_COM;
		}
	}
	catch(...)
	{
		if ((m_dwErrCode = GetLastError()) == ERROR_SUCCESS)
		{
			m_dwErrCode = ERR_UNEXPECTED_FOR_WIN;
		}
	}
	return bRetVal;
}

bool CComTools::CallMethod(LPCWSTR pszMethod, LP_SHORT2ARGVALUE pShort2ArgValue, variant_t& vtReturnValue, bool& bSetTypeResultAttr)
{
	bool bRetVal = false;
	HRESULT hr = E_FAIL;
	variant_t vtValue;
	bool bInConvert = false;
	const wchar_t* wpszParmName = NULL;
	try
	{
		if (m_spDisp.p == NULL)
		{
			_com_issue_error(E_INVALIDARG); //E_UNEXPECTED
		}
		
		clearError();
		m_uArgErr = 0;
		memset((LPVOID)&m_ExceptInfo, 0, sizeof(EXCEPINFO));
		bSetTypeResultAttr = false;

		// Get DispID & TypeInfo
		TESTHR(hr = m_spDisp->GetIDsOfNames(IID_NULL, (LPWSTR*)&pszMethod, 1, LOCALE_SYSTEM_DEFAULT, &m_dispid));
		TESTHR(hr = m_spDisp->GetTypeInfo(0, m_dispid, &m_spTypeInfo));
		// Get FuncDesc
		FUNCDESC* pFuncDesc = NULL;
		if ((hr = m_spDisp->QueryInterface(IID_ITypeInfo2,(void**)&m_spTypeInfo2)) == S_OK)
		{
			// From TypeInfo2
			UINT nIndex = 0;
			TESTHR(hr = m_spTypeInfo2->GetFuncIndexOfMemId((MEMBERID)m_dispid, INVOKE_FUNC, &nIndex)); 
			TESTHR(hr = m_spTypeInfo->GetFuncDesc(nIndex, &pFuncDesc));
		}
		else
		{
			// From TypeInfo
			TYPEATTR* pAttr = NULL;
			TESTHR(hr = m_spTypeInfo->GetTypeAttr(&pAttr));
			for (WORD wCnt = 0; wCnt < pAttr->cFuncs; wCnt++)
			{
				TESTHR(hr = m_spTypeInfo->GetFuncDesc((UINT)wCnt, &pFuncDesc));
				if (pFuncDesc->memid == m_dispid)
				{
					break;
				}
				m_spTypeInfo->ReleaseFuncDesc(pFuncDesc);
				pFuncDesc = NULL;
			}
			m_spTypeInfo->ReleaseTypeAttr(pAttr);
			pAttr = NULL;
		}
		if (pFuncDesc != NULL)
		{
			//bstr_t bstrFuncName;
			//TESTHR(hr = m_spTypeInfo->GetDocumentation(pFuncDesc->memid, &bstrFuncName.GetBSTR(), 0, 0, 0));
			// Init DispParams
			SHORT cParams = pFuncDesc->cParams;
			if (pFuncDesc->elemdescFunc.tdesc.vt == VT_VARIANT)
			{
				bSetTypeResultAttr = true;
			}
			clearDispParams();
			m_DispParams.cArgs = (UINT)cParams;
			if (m_DispParams.cArgs > 0)
			{
				m_DispParams.rgvarg = (VARIANTARG*)new VARIANTARG[m_DispParams.cArgs];
				if (m_DispParams.rgvarg == NULL)
				{
					_com_issue_error(E_OUTOFMEMORY);
				}
			}
			// Set parameters values (in reverse order)
			VARTYPE vtFrom = 0;
			VARTYPE vtTo = 0;
			TYPEDESC *ptdescSrc = NULL;
			SHORT2ARGVALUE::const_iterator theIter;
			int nItem = cParams;
			bInConvert = true;
			for (SHORT sParm = 0; sParm < cParams; sParm++)
			{
				VariantInit(&m_DispParams.rgvarg[--nItem]);
				// Find current parameter in pShort2ArgValue
				theIter = pShort2ArgValue->find(sParm);
				wpszParmName = (const wchar_t*)(*theIter).second.bstrName; 
				// Get current parameter value
				vtValue = (variant_t)(*theIter).second.vtValue;
				// Get type of parameter value from pShort2ArgValue
				if ((vtFrom = (*theIter).second.vt) == VT_EMPTY)
				{
					// If not found in pShort2ArgValue
					// get type of parameter value from TypeLib
					ptdescSrc = &pFuncDesc->lprgelemdescParam[sParm].tdesc;
					vtFrom = ptdescSrc->vt;
				}
				// Convert type of parameter value
				TypeDescToDWord(vtFrom, &vtTo, false);
				// Convert current parameter value
				TESTHR(hr = VariantChangeType(&m_DispParams.rgvarg[nItem], &vtValue.GetVARIANT(),
					VARIANT_NOVALUEPROP|VARIANT_NOUSEROVERRIDE|VARIANT_LOCALBOOL, vtTo));
			}
			bInConvert = false;
			m_spTypeInfo->ReleaseFuncDesc(pFuncDesc);
			// Invoke
			VariantInit(&m_vtReturnValue.GetVARIANT());
			TESTHR(hr = m_spDisp->Invoke(m_dispid, IID_NULL, LOCALE_USER_DEFAULT, DISPATCH_METHOD,
				&m_DispParams, &m_vtReturnValue.GetVARIANT(), &m_ExceptInfo, &m_uArgErr));  
			// Set result
			vtReturnValue = m_vtReturnValue;
			bRetVal = true;
		}
		else
		{
			_com_issue_error(E_INVALIDARG);
		}
	}
	catch(_com_error& e)
	{
		wchar_t buff[_MAX_PATH];
		wchar_t tmpbuff[_MAX_PATH];
		if ((m_hr = e.Error()) != S_OK)
		{
			m_bstrErrMsg = e.ErrorMessage();
			if (bInConvert)
			{
				swprintf(tmpbuff, L"\n%s = %s", wpszParmName, bstr_t(vtValue).GetBSTR());
				m_bstrErrMsg += tmpbuff;
			}
		}
		else
		{
			m_hr = ERR_UNEXPECTED_FOR_COM;
		}
		if (m_uArgErr != 0)
		{
			swprintf(buff, L"\n%s%d", ERR_EXEPT_ARG_LOC, m_uArgErr);
			m_bstrErrMsg += buff;
		}
		if (m_ExceptInfo.scode != 0)
		{
			swprintf(buff, L"\n%s%d", L"scode: ", m_ExceptInfo.scode);
			m_bstrErrMsg += buff;
		}
		if (m_ExceptInfo.wCode != 0)
		{
			swprintf(buff, L"\n%s%d", L"wCode: ", m_ExceptInfo.wCode);
			m_bstrErrMsg += buff;
		}
		if (m_ExceptInfo.bstrDescription != NULL)
		{
			swprintf(tmpbuff, L"%s", m_ExceptInfo.bstrDescription);
			if (wcslen(tmpbuff) > 0)
			{
				swprintf(buff, L"\n%s%s", ERR_EXEPT_DES_LOC, tmpbuff);
				m_bstrErrMsg += buff;
			}
		}
		if (m_ExceptInfo.bstrSource != NULL)
		{
			swprintf(tmpbuff, L"%s", m_ExceptInfo.bstrSource);
			if (wcslen(tmpbuff) > 0)
			{
				swprintf(buff, L"\n%s%s", ERR_EXEPT_SRC_LOC, tmpbuff);
				m_bstrErrMsg += buff;
			}
		}
		if (m_ExceptInfo.bstrHelpFile != NULL)
		{
			swprintf(tmpbuff, L"%s", m_ExceptInfo.bstrHelpFile);
			if (wcslen(tmpbuff) > 0)
			{
				swprintf(buff, L"\n%s%s", ERR_EXEPT_HLP_LOC, tmpbuff);
				m_bstrErrMsg += buff;
			}
		}
	}
	catch(...)
	{
		if ((m_dwErrCode = GetLastError()) == ERROR_SUCCESS)
		{
			if (bInConvert)
			{
				m_dwErrCode = ERROR_INVALID_DATA;
			}
			else
			{
				m_dwErrCode = ERR_UNEXPECTED_FOR_WIN;
			}
		}
	}
	clearDispParams();
	VariantClear(&m_vtReturnValue.GetVARIANT());
	return bRetVal;
}