#include "StdAfx.h"
#include "msxml2.h"
#include "XmlTools.h"

TYPEVARIANT CXmlTools::m_VariantTypes[] = 
{
	L"VT_EMPTY", VT_EMPTY,
	L"VT_NULL", VT_NULL,
    L"VT_I2", VT_I2,
    L"VT_I4", VT_I4,
    L"VT_R4", VT_R4,
    L"VT_R8", VT_R8,
    L"VT_CY", VT_CY,
    L"VT_DATE", VT_DATE,
    L"VT_BSTR", VT_BSTR,
    L"VT_DISPATCH", VT_DISPATCH,
    L"VT_ERROR", VT_ERROR,
    L"VT_BOOL", VT_BOOL,
    L"VT_VARIANT", VT_VARIANT,
    L"VT_UNKNOWN", VT_UNKNOWN,
    L"VT_DECIMAL", VT_DECIMAL,
    L"VT_I1", VT_I1,
    L"VT_UI1", VT_UI1,
    L"VT_UI2", VT_UI2,
    L"VT_UI4", VT_UI4,
    L"VT_I8", VT_I8,
    L"VT_UI8", VT_UI8,
    L"VT_INT", VT_INT,
    L"VT_UINT", VT_UINT,
    L"VT_VOID", VT_VOID,
    L"VT_HRESULT", VT_HRESULT,
    L"VT_PTR", VT_PTR,
    L"VT_SAFEARRAY", VT_SAFEARRAY,
    L"VT_CARRAY", VT_CARRAY,
    L"VT_USERDEFINED", VT_USERDEFINED,
    L"VT_LPSTR", VT_LPSTR,
    L"VT_LPWSTR", VT_LPWSTR,
    L"VT_FILETIME", VT_FILETIME,
    L"VT_BLOB", VT_BLOB,
    L"VT_STREAM", VT_STREAM,
    L"VT_STORAGE", VT_STORAGE,
    L"VT_STREAMED_OBJECT", VT_STREAMED_OBJECT,
    L"VT_STORED_OBJECT", VT_STORED_OBJECT,
    L"VT_BLOB_OBJECT", VT_BLOB_OBJECT,
    L"VT_CF", VT_CF,
    L"VT_CLSID", VT_CLSID,
    L"VT_VECTOR", VT_VECTOR,
    L"VT_ARRAY", VT_ARRAY,
    L"VT_BYREF", VT_BYREF,
    L"VT_RESERVED", VT_RESERVED,
    L"VT_ILLEGAL", VT_ILLEGAL,
    L"VT_ILLEGALMASKED", VT_ILLEGALMASKED,
    L"VT_TYPEMASK", VT_TYPEMASK
};

CXmlTools::CXmlTools(void)
{
	clearError();
}

CXmlTools::~CXmlTools(void)
{
	ReleaseAll();
}


void CXmlTools::ReleaseAll()
{
	if (m_mapSHORT2ARGVALUE.size())
	{
		m_mapSHORT2ARGVALUE.clear(); 
	}
	if (m_spXMLDOMDocument.p != NULL)
	{
		m_spXMLDOMDocument.Release(); 
	}
}

void CXmlTools::clearError()
{
	m_dwErrCode = ERROR_SUCCESS;
	m_errorCode 
		= m_filePosition 
		= m_lineNumber 
		= m_linePosition
		= m_hr
		= S_OK;
	m_reasonString
		= m_sourceString
		= m_urlString
		= m_bstrErrMsg
		= _bstr_t();
}

VARTYPE CXmlTools::getVariantType(const _bstr_t& bstrType)
{
	for(int i = 0; i < sizeof(CXmlTools::m_VariantTypes)/sizeof(TYPEVARIANT); i++)
	{
		if (CXmlTools::m_VariantTypes[i].bstrType == bstrType)
		{
			return CXmlTools::m_VariantTypes[i].vt; 
		}
	}
	return VT_EMPTY;
}

_bstr_t CXmlTools::getVariantType(const VARTYPE vt)
{
	for(int i = 0; i < sizeof(CXmlTools::m_VariantTypes)/sizeof(TYPEVARIANT); i++)
	{
		if (CXmlTools::m_VariantTypes[i].vt == vt)
		{
			return CXmlTools::m_VariantTypes[i].bstrType.copy(); 
		}
	}
	return L"VT_EMPTY";
}

bool CXmlTools::SetErrToDoc()
{
	bool bRetVal = false;
	HRESULT hr = S_OK;
	CComPtr<IXMLDOMParseError> spObjError;
	CComPtr<IXMLDOMNode> spComObj;
	CComPtr<IXMLDOMNode> spIDOMNode;
	CComPtr<IXMLDOMNode> spCurIDOMNode;
	CComPtr<IXMLDOMNode> spErrorNode;
	CComPtr<IXMLDOMNode> spXmlErrNode;
	CComPtr<IXMLDOMNode> spOleErrNode;
	CComPtr<IXMLDOMNode> spWin32ErrNode;
	CComPtr<IXMLDOMCDATASection> spDSNode;
	VARIANT vaTypeNodeElement = variant_t(NODE_ELEMENT).GetVARIANT();
	try
	{
		if (m_spXMLDOMDocument.p == NULL)
		{
			return bRetVal;
		}
		if (m_hr == 0L && m_errorCode == 0L && m_dwErrCode == 0)
		{
			return bRetVal;
		}
		// ComObj
		TESTHR(hr = m_spXMLDOMDocument->selectSingleNode(bstr_t(PATH_COMOBJ), &spComObj));
		if (hr != S_OK)
		{
			return bRetVal;
		}
		// Error
		AddNode(L"Error", spComObj, spErrorNode, hr, spIDOMNode, vaTypeNodeElement);
		// XmlErr
		if (m_errorCode != S_OK)
		{
			// XmlErr
			AddNode(L"XmlErr", spErrorNode, spXmlErrNode, hr, spIDOMNode, vaTypeNodeElement);
			// errorCode
			AddNode(L"errorCode", spXmlErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			TESTHR(hr = spCurIDOMNode->put_text(bstr_t(m_errorCode).GetBSTR()));
			spCurIDOMNode.Release(); 
			// filePosition
			AddNode(L"filePosition", spXmlErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			TESTHR(hr = spCurIDOMNode->put_text(bstr_t(m_filePosition).GetBSTR()));
			spCurIDOMNode.Release(); 
			// lineNumber
			AddNode(L"lineNumber", spXmlErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			TESTHR(hr = spCurIDOMNode->put_text(bstr_t(m_lineNumber).GetBSTR()));
			spCurIDOMNode.Release(); 
			// linePosition
			AddNode(L"linePosition", spXmlErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			TESTHR(hr = spCurIDOMNode->put_text(bstr_t(m_linePosition).GetBSTR()));
			spCurIDOMNode.Release(); 
			// reasonString
			AddNode(L"reasonString", spXmlErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			AddDS(m_reasonString, spCurIDOMNode, spDSNode, hr, spIDOMNode);			
			spCurIDOMNode.Release(); 
			// sourceString
			AddNode(L"sourceString", spXmlErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			AddDS(m_sourceString, spCurIDOMNode, spDSNode, hr, spIDOMNode);			
			spCurIDOMNode.Release(); 
			// urlString
			AddNode(L"urlString", spXmlErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			AddDS(m_urlString, spCurIDOMNode, spDSNode, hr, spIDOMNode);			
			spCurIDOMNode.Release(); 
			bRetVal = true;
		}
		// OleErr
		if (m_hr != S_OK)
		{
			// OleErr
			AddNode(L"OleErr", spErrorNode, spOleErrNode, hr, spIDOMNode, vaTypeNodeElement);
			// Code
			AddNode(L"Code", spOleErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			TESTHR(hr = spCurIDOMNode->put_text(bstr_t(m_hr).GetBSTR()));
			spCurIDOMNode.Release(); 
			// Description
			AddNode(L"Description", spOleErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			AddDS(m_bstrErrMsg, spCurIDOMNode, spDSNode, hr, spIDOMNode);			
			spCurIDOMNode.Release(); 
			bRetVal = true;
		}
		// Win32Err
		if (m_dwErrCode != ERROR_SUCCESS)
		{
			// Win32Err
			AddNode(L"Win32Err", spErrorNode, spWin32ErrNode, hr, spIDOMNode, vaTypeNodeElement);
			// Code
			AddNode(L"Code", spWin32ErrNode, spCurIDOMNode, hr, spIDOMNode, vaTypeNodeElement);
			TESTHR(hr = spCurIDOMNode->put_text(bstr_t(m_dwErrCode).GetBSTR()));
			spCurIDOMNode.Release(); 
		}
	}
	catch(_com_error& e)
	{
		m_hr = e.Error(); 
		m_bstrErrMsg = e.ErrorMessage();
	}
	catch(IXMLDOMParseError* e)
	{
		e->get_errorCode(&m_errorCode); 
		e->get_filepos(&m_filePosition); 
		e->get_line(&m_lineNumber); 
		e->get_linepos(&m_linePosition); 
		e->get_reason(&m_reasonString.GetBSTR());
		e->get_srcText(&m_sourceString.GetBSTR());
		e->get_url(&m_urlString.GetBSTR());
		e->Release(); 
	}
	catch(...)
	{
		m_bstrErrMsg = ERR_UNEXPECTED_LOC;
		m_hr = E_FAIL; 
	}
	if (spDSNode.p != NULL)
	{
		spDSNode.Release(); 
	}
	if (spCurIDOMNode.p != NULL)
	{
		spCurIDOMNode.Release(); 
	}
	if (spIDOMNode.p != NULL)
	{
		spIDOMNode.Release(); 
	}
	if (spObjError.p != NULL)
	{
		spObjError.Release(); 
	}
	if (spXmlErrNode.p != NULL)
	{
		spXmlErrNode.Release(); 
	}
	if (spOleErrNode.p != NULL)
	{
		spOleErrNode.Release(); 
	}
	if (spOleErrNode.p != NULL)
	{
		spOleErrNode.Release(); 
	}
	if (spWin32ErrNode.p != NULL)
	{
		spWin32ErrNode.Release(); 
	}
	if (spComObj.p != NULL)
	{
		spComObj.Release(); 
	}
	return bRetVal;
}

bool CXmlTools::loasXML(_bstr_t& bstrXml)
{
	bool bRetVal = false;
	if (bstrXml.length() == 0)
	{
		return bRetVal;
	}	
	HRESULT hr = S_OK;
	VARIANT_BOOL bResVal = VARIANT_FALSE;
	long nValue = 0;
	bstr_t bstrValue;
	CComPtr<IXMLDOMNode> spIDOMNode;
	CComPtr<IXMLDOMNode> spIDOMNodeAttr;
	CComPtr<IXMLDOMParseError> spObjError;
	CComPtr<IXMLDOMNodeList> spIDOMNodeList;
	CComPtr<IXMLDOMNodeList> spIDOMNodeListParam;
	CComPtr<IXMLDOMNamedNodeMap> spDOMNamedNodeMap;
	clearError();
	try
	{
		if (m_spXMLDOMDocument.p == NULL)
		{
			TESTHR(hr = m_spXMLDOMDocument.CoCreateInstance(CLSID_FreeThreadedDOMDocument40,
				0, CLSCTX_INPROC_SERVER));
		}
		TESTHR(hr = m_spXMLDOMDocument->put_async(VARIANT_FALSE));
		TESTHR(hr = m_spXMLDOMDocument->put_validateOnParse(VALIDATEONPARS)); 
		TESTHR(hr = m_spXMLDOMDocument->loadXML(bstrXml.GetBSTR(), &bResVal));
		if (bResVal != VARIANT_TRUE)
		{
			TESTHR(hr = m_spXMLDOMDocument->get_parseError(&spObjError));
			throw((IXMLDOMParseError*)spObjError);
		}
		// ProgID
		TESTHR(hr = m_spXMLDOMDocument->selectSingleNode(bstr_t(PATH_PROGID), &spIDOMNode));
		TESTHR(hr = spIDOMNode->get_text(&m_bstrProgID.GetBSTR()));
		spIDOMNode.Release(); 
		// Method
		TESTHR(hr = m_spXMLDOMDocument->selectSingleNode(bstr_t(PATH_METHOD), &spIDOMNode));
		TESTHR(hr = spIDOMNode->get_text(&m_bstrMethod.GetBSTR())); 
		spIDOMNode.Release();
		// Params
		TESTHR(hr = m_spXMLDOMDocument->selectSingleNode(bstr_t(PATH_PARAMS), &spIDOMNode));
		if (hr == S_OK)
		{
			TESTHR(hr = spIDOMNode->get_childNodes(&spIDOMNodeList)); 
			spIDOMNode.Release();
			long listLength = 0L;
			TESTHR(hr = spIDOMNodeList->get_length(&listLength));
			if (listLength > 0L)
			{
				VARTYPE vt = VT_EMPTY;
				bstr_t bstrCurType;
				bstr_t bstrCurName;
				bstr_t bstrCurValue;
				bstr_t bstrNodeName;
				const wchar_t* wpzsNodeName = NULL;
				long nLengtParams = 0L;
				for(long nItem = 0; nItem < listLength; nItem++)
				{
					// For each Param
					vt = VT_EMPTY;
					bstrCurType = CXmlTools::m_VariantTypes[0].bstrType.copy(); // VT_EMPTY
					bstrNodeName = bstrCurName = bstrCurValue = bstr_t();
					TESTHR(hr = spIDOMNodeList->get_item(nItem, &spIDOMNode));
					// Get attributes
					if ((hr = spIDOMNode->get_attributes(&spDOMNamedNodeMap)) == S_OK)
					{
						// Get 'name' attribute
						if ((hr = spDOMNamedNodeMap->getNamedItem(L"name", &spIDOMNodeAttr)) == S_OK)
						{
							TESTHR(hr = spIDOMNodeAttr->get_text(&bstrCurName.GetBSTR()));
							spIDOMNodeAttr.Release(); 
						}
						// Get 'type' attribute
						if ((hr = spDOMNamedNodeMap->getNamedItem(L"type", &spIDOMNodeAttr)) == S_OK)
						{
							TESTHR(hr = spIDOMNodeAttr->get_text(&bstrCurType.GetBSTR()));
							spIDOMNodeAttr.Release(); 
						}
						spDOMNamedNodeMap.Release(); 
					}
					// Get VARTYPE for 'type' attribute (convert from string)
					if (bstrCurType != CXmlTools::m_VariantTypes[0].bstrType)
					{
						vt = getVariantType(bstrCurType);
					}
					TESTHR(hr = spIDOMNode->get_childNodes(&spIDOMNodeListParam)); 
					spIDOMNode.Release();
					if (hr == S_OK)
					{
						// Get Value || ValueArray
						nLengtParams = 0L;
						TESTHR(hr = spIDOMNodeListParam->get_length(&nLengtParams));
						if (nLengtParams > 0L)
						{
							// For first child only
							TESTHR(hr = spIDOMNodeListParam->get_item(0, &spIDOMNode));
							if (hr == S_OK)
							{
								// Get nodeName
								TESTHR(hr = spIDOMNode->get_nodeName(&bstrNodeName.GetBSTR()));
								wpzsNodeName = (const wchar_t*)bstrNodeName;
								if (wcscmp(wpzsNodeName, L"Value") == 0 
									|| wcscmp(wpzsNodeName, L"#text") == 0)
								{
									// Value node
									TESTHR(hr = spIDOMNode->get_text(&bstrCurValue.GetBSTR())); 
								}if (wcscmp(wpzsNodeName, L"ValueArray") == 0)
								{
									// ValueArray node - Not impl yet
									_com_issue_error(E_NOTIMPL); //E_INVALIDARG E_UNEXPECTED
								}
								spIDOMNode.Release();
							}
						}
					}
					if (bstrCurValue.length() ==0)
					{
						// Not found Value
						_com_issue_error(E_INVALIDARG); //E_UNEXPECTED
					}
					m_mapSHORT2ARGVALUE.insert(SHORT2ARGVALUE::value_type((SHORT)nItem,
						_ARGVALUE(bstrCurName, _variant_t(bstrCurValue.GetBSTR()), vt)));
					spIDOMNodeListParam.Release(); 
				}
			}
		}
		bRetVal = true;
	}
	catch(IXMLDOMParseError* e)
	{
		e->get_errorCode(&m_errorCode); 
		e->get_filepos(&m_filePosition); 
		e->get_line(&m_lineNumber); 
		e->get_linepos(&m_linePosition); 
		e->get_reason(&m_reasonString.GetBSTR());
		e->get_srcText(&m_sourceString.GetBSTR());
		e->get_url(&m_urlString.GetBSTR());
		e->Release(); 
		SetErrToDoc();
	}
	catch(_com_error& e)
	{
		if ((m_hr = e.Error()) != S_OK)
		{
			m_bstrErrMsg = e.ErrorMessage();
		}
		else
		{
			m_hr = ERR_UNEXPECTED_FOR_COM;
		}
		SetErrToDoc();
	}
	catch(...)
	{
		if ((m_dwErrCode = GetLastError()) == ERROR_SUCCESS)
		{
			m_dwErrCode = ERR_UNEXPECTED_FOR_WIN;
		}
		SetErrToDoc();
	}
	if (spIDOMNode.p != NULL)
	{
		spIDOMNode.Release(); 
	}
	if (spIDOMNodeAttr.p != NULL)
	{
		spIDOMNodeAttr.Release(); 
	}
	if (spDOMNamedNodeMap.p != NULL)
	{
		spDOMNamedNodeMap.Release(); 
	}
	if (spIDOMNodeListParam.p != NULL)
	{
		spIDOMNodeListParam.Release(); 
	}
	if (spObjError.p != NULL)
	{
		spObjError.Release(); 
	}
	return bRetVal;
}

bool CXmlTools::getDocAsXML(_bstr_t& bstrXml)
{
	bool bRetVal = false;
	HRESULT hr = S_OK;
	clearError();
	try
	{
		if (m_spXMLDOMDocument.p == NULL)
		{
			return bRetVal;
		}
		TESTHR(hr = m_spXMLDOMDocument->get_xml(&bstrXml.GetBSTR()));
		bRetVal = true;
		wchar_t* lpStart = bstrXml.Detach();
		wchar_t* lpPiEnd = wcsstr(lpStart, L"?>");
		if (lpPiEnd != NULL)
		{
			// Insert processing-instruction: 'encoding' & 'xml-stylesheet'
			bstr_t bstrContent = lpPiEnd + sizeof(wchar_t);
			bstrXml = bstr_t(XMLPI_ENCODING)
				+ bstr_t(XMLPI_STYLESHEET)
				+ bstrContent;
		}
	}
	catch(IXMLDOMParseError* e)
	{
		e->get_errorCode(&m_errorCode); 
		e->get_filepos(&m_filePosition); 
		e->get_line(&m_lineNumber); 
		e->get_linepos(&m_linePosition); 
		e->get_reason(&m_reasonString.GetBSTR());
		e->get_srcText(&m_sourceString.GetBSTR());
		e->get_url(&m_urlString.GetBSTR());
		e->Release(); 
		SetErrToDoc();
	}
	catch(_com_error& e)
	{
		if ((m_hr = e.Error()) != S_OK)
		{
			m_bstrErrMsg = e.ErrorMessage();
		}
		else
		{
			m_hr = ERR_UNEXPECTED_FOR_COM;
		}
		SetErrToDoc();
	}
	catch(...)
	{
		if ((m_dwErrCode = GetLastError()) == ERROR_SUCCESS)
		{
			m_dwErrCode = ERR_UNEXPECTED_FOR_WIN;
		}
		SetErrToDoc();
	}
	return bRetVal;
}

bool CXmlTools::putOleError(HRESULT hr, const bstr_t& bstrErrMsg, DWORD dwErrCode)
{
	m_hr = hr;
	m_bstrErrMsg = bstrErrMsg.copy();
	m_dwErrCode = dwErrCode;
	return SetErrToDoc();
}

bool CXmlTools::putResult(variant_t vtResult, bool bSetTypeResultAttr)
{
	bool bRetVal = false;
	HRESULT hr = S_OK;
	CComPtr<IXMLDOMNode> spComObj;
	CComPtr<IXMLDOMNode> spIDOMNode;
	CComPtr<IXMLDOMNode> spResultsNode;
	CComPtr<IXMLDOMNode> spResultNode;
	CComPtr<IXMLDOMNode> spValueNode;
	CComPtr<IXMLDOMAttribute> spAttribute;
	CComPtr<IXMLDOMNamedNodeMap> spNamedNodeMap;
	CComPtr<IXMLDOMCDATASection> spDSNode;
	VARIANT vaTypeNodeElement = variant_t(NODE_ELEMENT).GetVARIANT();
	try
	{
		if (m_spXMLDOMDocument.p == NULL)
		{
			return bRetVal;
		}
		bstr_t bstrResult = bstr_t(vtResult);
		if (bstrResult.length() == 0)
		{
			return bRetVal;
		}
		// ComObj
		TESTHR(hr = m_spXMLDOMDocument->selectSingleNode(bstr_t(PATH_COMOBJ), &spComObj));
		if (hr != S_OK)
		{
			return bRetVal;
		}
		// Results
		AddNode(L"Results", spComObj, spResultsNode, hr, spIDOMNode, vaTypeNodeElement);
		// Result
		AddNode(L"Result", spResultsNode, spResultNode, hr, spIDOMNode, vaTypeNodeElement);
		if (bSetTypeResultAttr)
		{
			// Attribute 'type'
			TESTHR(hr = m_spXMLDOMDocument->createAttribute(L"type", &spAttribute));
			TESTHR(hr = spAttribute->put_text(getVariantType(vtResult.vt).GetBSTR())); 
			TESTHR(hr = spResultNode->get_attributes(&spNamedNodeMap));
			TESTHR(hr = spNamedNodeMap->setNamedItem((CComPtr<IXMLDOMNode>)spAttribute, &spIDOMNode));
			spIDOMNode.Release(); 
		}
		// Value
		AddNode(L"Value", spResultNode, spValueNode, hr, spIDOMNode, vaTypeNodeElement);
		AddDS(bstrResult, spValueNode, spDSNode, hr, spIDOMNode);			
		spValueNode.Release(); 
		bRetVal = true;
	}
	catch(IXMLDOMParseError* e)
	{
		e->get_errorCode(&m_errorCode); 
		e->get_filepos(&m_filePosition); 
		e->get_line(&m_lineNumber); 
		e->get_linepos(&m_linePosition); 
		e->get_reason(&m_reasonString.GetBSTR());
		e->get_srcText(&m_sourceString.GetBSTR());
		e->get_url(&m_urlString.GetBSTR());
		e->Release(); 
		SetErrToDoc();
	}
	catch(_com_error& e)
	{
		if ((m_hr = e.Error()) != S_OK)
		{
			m_bstrErrMsg = e.ErrorMessage();
		}
		else
		{
			m_hr = ERR_UNEXPECTED_FOR_COM;
		}
		SetErrToDoc();
	}
	catch(...)
	{
		if ((m_dwErrCode = GetLastError()) == ERROR_SUCCESS)
		{
			m_dwErrCode = ERR_UNEXPECTED_FOR_WIN;
		}
		SetErrToDoc();
	}
	if (spDSNode.p != NULL)
	{
		spDSNode.Release(); 
	}
	if (spAttribute.p != NULL)
	{
		spAttribute.Release(); 
	}
	if (spNamedNodeMap.p != NULL)
	{
		spNamedNodeMap.Release(); 
	}
	if (spValueNode.p != NULL)
	{
		spValueNode.Release(); 
	}
	if (spResultNode.p != NULL)
	{
		spResultNode.Release(); 
	}
	if (spResultsNode.p != NULL)
	{
		spResultsNode.Release(); 
	}
	if (spIDOMNode.p != NULL)
	{
		spIDOMNode.Release(); 
	}
	if (spComObj.p != NULL)
	{
		spComObj.Release(); 
	}
	return bRetVal;
}

