<?xml version="1.0" encoding="UTF-8" ?>
<!-- file: Result.xslt -->
<xsl:stylesheet version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:msxsl="urn:schemas-microsoft-com:xslt"
	xmlns:user="myextfuncs"
	xmlns="http://www.w3.org/TR/html4"
	exclude-result-prefixes="xsl msxsl user">

	<xsl:output method="html" 
		encoding="windows-1251" 
		version="4.0"
		indent="no" 
		doctype-public="-//W3C//DTD HTML 4.0 Transitional//EN" />
	
	<xsl:strip-space elements="*" />
	
	<xsl:template match="/">
		<HTML>
			<META name="vs_targetSchema" content="http://schemas.microsoft.com/intellisense/ie5" />
			<META http-equiv="Content-Type" content="text/html; charset=windows-1251" />
			<META http-equiv="Content-Language" content="ru" />
			<LINK type="text/css" rel="stylesheet" href="Result.css" />
			<TITLE>Результат обращения к VFP-COM компоненте</TITLE>
			<BODY class="body_page">
				<xsl:apply-templates select=".//ComObj" />
			</BODY>
		</HTML>
	</xsl:template>

	<xsl:template match="ComObj">
		<H4>Данные вызванной компоненты</H4>
		<TABLE class="tblComInfo" width="50%">
			<TR>
				<TH class="tblComInfo">Название</TH>
				<TH class="tblComInfo">Значение</TH>
			</TR>
			<TR>
				<TD class="tdName">Имя компоненты</TD>
				<TD class="tdValue"><xsl:value-of select="ProgID" /></TD>
			</TR>
			<TR>
				<TD class="tdName">Вызываемый метод</TD>
				<TD class="tdValue"><xsl:value-of select="Method" /></TD>
			</TR>
		</TABLE>
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="ProgID" />
	<xsl:template match="Method" />
	
	<xsl:template match="Params">
		<H4>Значения параметров при обращении</H4>
		<TABLE class="tblComInfo" width="50%">
			<TR>
				<TH class="tblComInfo">Название</TH>
				<TH class="tblComInfo">Значение</TH>
				<TH class="tblComInfo">Тип</TH>
			</TR>
			<xsl:apply-templates />
		</TABLE>
	</xsl:template>

	<xsl:template match="Param">
		<TR>
			<TD class="tdName">
				<xsl:if test="@name">
					<xsl:value-of select="@name" />
				</xsl:if>	
			</TD>
			<TD class="tdValue"><xsl:value-of select="Value" /></TD>
			<TD class="tdValue">
				<xsl:if test="@type">
					<xsl:value-of select="@type" />
				</xsl:if>	
			</TD>
		</TR>
	</xsl:template>
	
	<xsl:template match="Results">
		<xsl:if test="count(Result)>0 and string-length(Result) > 0">
			<H4>Результаты выполнения</H4>
			<TABLE class="tblComInfo" width="90%">
				<TR>
					<TH class="tblComInfo">Возвращённое значение</TH>
					<TH class="tblComInfo">Тип</TH>
				</TR>
				<xsl:apply-templates />
			</TABLE>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="Result">
		<TR>
			<TD class="tdValue"><xsl:value-of select="Value" disable-output-escaping = "yes" /></TD>
			<TD class="tdValue">
				<xsl:if test="@type">
					<xsl:value-of select="@type" />
				</xsl:if>	
			</TD>
		</TR>	
	</xsl:template>
	
	<xsl:template match="Value" />
	
	<xsl:template match="Error">
		<xsl:if test="string-length(Win32Err/Code)>0 or string-length(OleErr/Code) > 0 or string-length(XmlErr/errorCode)> 0">
			<H4 style="color: red;" >Информация о возникших ошибках</H4>
			<xsl:apply-templates />
		</xsl:if>
	</xsl:template>

	<xsl:template match="Win32Err">
		<xsl:if test="Code">
			<DIV><SPAN class="section_title">Код ошибки Win32API: </SPAN><xsl:value-of select="Code" disable-output-escaping = "yes" /></DIV>
		</xsl:if>
	</xsl:template>

	<xsl:template match="OleErr">
		<xsl:if test="Code">
			<DIV><SPAN class="section_title">Информация об OLE ошибке:</SPAN></DIV>
			<TABLE class="tblComInfo" width="90%">
				<TR>
					<TH class="tblComInfo">Код</TH>
					<TH class="tblComInfo">Пояснения</TH>
				</TR>
				<TR>
					<TD class="tdName"><xsl:value-of select="Code" disable-output-escaping = "yes" /></TD>
					<TD class="tdValue"><xsl:value-of select="Description" disable-output-escaping = "yes" /></TD>
				</TR>
			</TABLE>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="Code" />
	<xsl:template match="Description" />

	<xsl:template match="XmlErr">
		<xsl:if test="Code">
			<DIV><SPAN class="section_title">Информация об ошибке в XML:</SPAN></DIV>
			<TABLE class="tblComInfo" width="90%">
				<TR>
					<TH class="tblComInfo">Название парамера</TH>
					<TH class="tblComInfo">Значение</TH>
				</TR>
				<TR>
					<TD class="tdName">Код ошибки</TD>
					<TD class="tdValue"><xsl:value-of select="errorCode" disable-output-escaping = "yes" /></TD>
				</TR>
				<TR>
					<TD class="tdName">Позиция в файле</TD>
					<TD class="tdValue"><xsl:value-of select="filePosition" disable-output-escaping = "yes" /></TD>
				</TR>
				<TR>
					<TD class="tdName">Номер строки</TD>
					<TD class="tdValue"><xsl:value-of select="lineNumber" disable-output-escaping = "yes" /></TD>
				</TR>
				<TR>
					<TD class="tdName">Позиция в строке</TD>
					<TD class="tdValue"><xsl:value-of select="linePosition" disable-output-escaping = "yes" /></TD>
				</TR>
				<TR>
					<TD class="tdName">Пояснения к ошибке</TD>
					<TD class="tdValue"><xsl:value-of select="reasonString" disable-output-escaping = "yes" /></TD>
				</TR>
				<TR>
					<TD class="tdName">Текст в файле</TD>
					<TD class="tdValue"><xsl:value-of select="sourceString" disable-output-escaping = "yes" /></TD>
				</TR>
				<TR>
					<TD class="tdName">Название файла</TD>
					<TD class="tdValue"><xsl:value-of select="urlString" disable-output-escaping = "yes" /></TD>
				</TR>
			</TABLE>
		</xsl:if>
	</xsl:template>

	<xsl:template match="errorCode" />
	<xsl:template match="filePosition" />
	<xsl:template match="lineNumber" />
	<xsl:template match="linePosition" />
	<xsl:template match="reasonString" />
	<xsl:template match="sourceString" />
	<xsl:template match="urlString" />

</xsl:stylesheet>